# Contributing

The backtrace-sca service can be extended to ingest defects from additional static code analysis tools using the Adaptor abstract class.

1. Build an adaptor

     - Extend the AdaptorName type in `src/models/adaptor.ts` with the name of your adaptor.

    ```javascript
        export type AdaptorName = 'parasoft'|'clang-analyzer'|'my-new-adaptor';
     ```

     - Create a new class in `src/application/adaptors` that extends the Adaptor class (see `src/application/adaptors/Adaptor.ts` for more info, including helper methods available in the class).

     ```javascript
        import { Adaptor } from './Adaptor';
        import { IReportObject } from '../../models/report';
        import { IAdaptorOptions } from '../../models/adaptor';

        export class MyNewAdaptor extends Adaptor {
            constructor(options: IAdaptorOptions) {
            super(options, 'my-new-adaptor');
            }
            public async generateReports(): Promise<IReportObject[]> {
                // Your implmentation processing this.options.data into report(s)
                return [report];
            }
            public close() {
                // nothing here
            }
        }
     ```

    - Update `src/application/classifierMap.json` to include your adaptor name as a key. The value is an object where keys are features potentially identified by your static code analysis tool and the corresponding value is an array of the Backtrace classifiers that should get applied to defects with that feature.   

    ```json
        ...
        "my-new-adaptor": {
            "use-after-free": [
                "invalid-free"
            ],
            "memory-leak": [
                "DirectLeak"
            ], 
            "dereference-of-null-pointer": [
                "null"
            ]
        }
    ```

3. Update the submission route to use your adaptor

    - In `main.ts`, add a new case to the switch statement in the POST route `/api/sca/submit/:format`. The case condition is the name of your adaptor and the case statement should assign the variable `adaptor` to a new instance of your adaptor. 

    ```typescript
        import { MyNewAdaptor } from './application/adaptors/MyNewAdaptor';
        ...
        case 'my-new-adaptor':
            adaptor = new MyNewAdaptor(options);
            break;
        ...   
    ```
    - Update `availableFormats` to include a condition for your adaptor name.
    ```
    const availableFormats = service.configuration.formats.filter((format: string) => {
        return format === 'parasoft' || format === 'clang-analyzer' || format === 'my-new-adaptor';
    });
    ```

4. Update `sca.conf` to include your adaptor.

    The `formats` property lists the adaptor versions available to the end-user. It is used for gating adapators. If your adaptor should be available in the default version of the SCA service, you can commit it to the local `sca.conf`. If it has gating restrictions, you may want to commit it only to specific target config files via Salt. 

    ```json
        {
            ...
            "formats": ["clang-analyzer", "my-new-adaptor"],
            ...
        }
    ```
    

